@@ -194,23 +194,32 @@ static int calc_stdin (hashcat_ctx_t *hashcat_ctx, hc_device_param_t *device_par
     memset (device_param->pws_comp, 0, device_param->size_pws_comp);
     memset (device_param->pws_idx,  0, device_param->size_pws_idx);
 
+    #define DISABLE_READ_TIMEOUT_AFTER 1000
+
+    int selects_returned = 0;
+
     while (device_param->pws_cnt < device_param->kernel_power)
     {
-      const int rc_select = select_read_timeout_console (1);
-
-      if (rc_select == -1) break;
-
-      if (rc_select == 0)
+      if (selects_returned < DISABLE_READ_TIMEOUT_AFTER)
       {
-        if (status_ctx->run_thread_level1 == false) break;
+        const int rc_select = select_read_timeout_console (1);
 
-        status_ctx->stdin_read_timeout_cnt++;
+        if (rc_select == -1) break;
 
-        continue;
+        if (rc_select == 0)
+        {
+          if (status_ctx->run_thread_level1 == false) break;
+
+          status_ctx->stdin_read_timeout_cnt++;
+
+          continue;
+        }
+
+        status_ctx->stdin_read_timeout_cnt = 0;
+
+        selects_returned++;
       }
 
-      status_ctx->stdin_read_timeout_cnt = 0;
-
       char *line_buf = fgets (buf, HCBUFSIZ_LARGE - 1, stdin);
 
       if (line_buf == NULL) break;
@@ -298,7 +307,7 @@ static int calc_stdin (hashcat_ctx_t *hashcat_ctx, hc_device_param_t *device_par
       return -1;
     }
 
-    if (run_cracker (hashcat_ctx, device_param, device_param->pws_cnt) == -1)
+    if (run_cracker (hashcat_ctx, device_param, -1, device_param->pws_cnt) == -1) // no pws_pos?
     {
       hcfree (buf);
 
@@ -312,11 +321,13 @@ static int calc_stdin (hashcat_ctx_t *hashcat_ctx, hc_device_param_t *device_par
     if (device_param->speed_only_finish == true) break;
   }
 
-  device_param->kernel_accel_prev = device_param->kernel_accel;
-  device_param->kernel_loops_prev = device_param->kernel_loops;
+  device_param->kernel_accel_prev   = device_param->kernel_accel;
+  device_param->kernel_loops_prev   = device_param->kernel_loops;
+  device_param->kernel_threads_prev = device_param->kernel_threads;
 
-  device_param->kernel_accel = 0;
-  device_param->kernel_loops = 0;
+  device_param->kernel_accel   = 0;
+  device_param->kernel_loops   = 0;
+  device_param->kernel_threads = 0;
 
   if (iconv_enabled == true)
   {
@@ -347,7 +358,12 @@ HC_API_CALL void *thread_calc_stdin (void *p)
 
   if (device_param->is_cuda == true)
   {
-    if (hc_cuCtxSetCurrent (hashcat_ctx, device_param->cuda_context) == -1) return NULL;
+    if (hc_cuCtxPushCurrent (hashcat_ctx, device_param->cuda_context) == -1) return NULL;
+  }
+
+  if (device_param->is_hip == true)
+  {
+    if (hc_hipCtxPushCurrent (hashcat_ctx, device_param->hip_context) == -1) return NULL;
   }
 
   if (calc_stdin (hashcat_ctx, device_param) == -1)
@@ -357,6 +373,16 @@ HC_API_CALL void *thread_calc_stdin (void *p)
     status_ctx->devices_status = STATUS_ERROR;
   }
 
+  if (device_param->is_cuda == true)
+  {
+    if (hc_cuCtxPopCurrent (hashcat_ctx, &device_param->cuda_context) == -1) return NULL;
+  }
+
+  if (device_param->is_hip == true)
+  {
+    if (hc_hipCtxPopCurrent (hashcat_ctx, &device_param->hip_context) == -1) return NULL;
+  }
+
   return NULL;
 }
 
@@ -650,7 +676,7 @@ static int calc (hashcat_ctx_t *hashcat_ctx, hc_device_param_t *device_param)
             return -1;
           }
 
-          if (run_cracker (hashcat_ctx, device_param, pws_cnt) == -1)
+          if (run_cracker (hashcat_ctx, device_param, -1, pws_cnt) == -1)
           {
             hc_fclose (&extra_info_straight.fp);
 
@@ -963,7 +989,7 @@ static int calc (hashcat_ctx_t *hashcat_ctx, hc_device_param_t *device_param)
             return -1;
           }
 
-          if (run_cracker (hashcat_ctx, device_param, pws_cnt) == -1)
+          if (run_cracker (hashcat_ctx, device_param, -1, pws_cnt) == -1)
           {
             hc_fclose (&extra_info_combi.base_fp);
             hc_fclose (&extra_info_combi.combs_fp);
@@ -1202,7 +1228,7 @@ static int calc (hashcat_ctx_t *hashcat_ctx, hc_device_param_t *device_param)
         if (pws_cnt)
         {
           if (run_copy    (hashcat_ctx, device_param, pws_cnt) == -1) return -1;
-          if (run_cracker (hashcat_ctx, device_param, pws_cnt) == -1) return -1;
+          if (run_cracker (hashcat_ctx, device_param, -1, pws_cnt) == -1) return -1;
 
           #ifdef WITH_BRAIN
           if (user_options->brain_client == true)
@@ -1274,7 +1300,7 @@ static int calc (hashcat_ctx_t *hashcat_ctx, hc_device_param_t *device_param)
         device_param->pws_cnt = work;
 
         if (run_copy    (hashcat_ctx, device_param, device_param->pws_cnt) == -1) return -1;
-        if (run_cracker (hashcat_ctx, device_param, device_param->pws_cnt) == -1) return -1;
+        if (run_cracker (hashcat_ctx, device_param, -1, device_param->pws_cnt) == -1) return -1;
 
         device_param->pws_cnt = 0;
 
@@ -1406,6 +1432,14 @@ static int calc (hashcat_ctx_t *hashcat_ctx, hc_device_param_t *device_param)
               line_len = (u32) rule_len_out;
             }
 
+            if (user_options->attack_mode == ATTACK_MODE_ASSOCIATION)
+            {
+              // we can't reject password base on length in -a 9 because it will bring the schedule out of sync
+              // therefore we render it defective so the other candidates survive
+
+              line_len = MIN (line_len, hashconfig->pw_max);
+            }
+
             if (attack_kern == ATTACK_KERN_STRAIGHT)
             {
               if ((line_len < hashconfig->pw_min) || (line_len > hashconfig->pw_max))
@@ -1479,7 +1513,7 @@ static int calc (hashcat_ctx_t *hashcat_ctx, hc_device_param_t *device_param)
             return -1;
           }
 
-          if (run_cracker (hashcat_ctx, device_param, pws_cnt) == -1)
+          if (run_cracker (hashcat_ctx, device_param, device_param->words_off, pws_cnt) == -1)
           {
             if (attack_mode == ATTACK_MODE_COMBI) hc_fclose (&device_param->combs_fp);
 
@@ -1547,11 +1581,13 @@ static int calc (hashcat_ctx_t *hashcat_ctx, hc_device_param_t *device_param)
     }
   }
 
-  device_param->kernel_accel_prev = device_param->kernel_accel;
-  device_param->kernel_loops_prev = device_param->kernel_loops;
+  device_param->kernel_accel_prev   = device_param->kernel_accel;
+  device_param->kernel_loops_prev   = device_param->kernel_loops;
+  device_param->kernel_threads_prev = device_param->kernel_threads;
 
-  device_param->kernel_accel = 0;
-  device_param->kernel_loops = 0;
+  device_param->kernel_accel   = 0;
+  device_param->kernel_loops   = 0;
+  device_param->kernel_threads = 0;
 
   return 0;
 }
@@ -1573,7 +1609,12 @@ HC_API_CALL void *thread_calc (void *p)
 
   if (device_param->is_cuda == true)
   {
-    if (hc_cuCtxSetCurrent (hashcat_ctx, device_param->cuda_context) == -1) return NULL;
+    if (hc_cuCtxPushCurrent (hashcat_ctx, device_param->cuda_context) == -1) return NULL;
+  }
+
+  if (device_param->is_hip == true)
+  {
+    if (hc_hipCtxPushCurrent (hashcat_ctx, device_param->hip_context) == -1) return NULL;
   }
 
   if (calc (hashcat_ctx, device_param) == -1)
@@ -1583,5 +1624,15 @@ HC_API_CALL void *thread_calc (void *p)
     status_ctx->devices_status = STATUS_ERROR;
   }
 
+  if (device_param->is_cuda == true)
+  {
+    if (hc_cuCtxPopCurrent (hashcat_ctx, &device_param->cuda_context) == -1) return NULL;
+  }
+
+  if (device_param->is_hip == true)
+  {
+    if (hc_hipCtxPopCurrent (hashcat_ctx, &device_param->hip_context) == -1) return NULL;
+  }
+
   return NULL;
 }
